@@ -1,6 +1,6 @@
 # -*-python-*-
 #
-# Copyright (C) 1999-2008 The ViewCVS Group. All Rights Reserved.
+# Copyright (C) 1999-2009 The ViewCVS Group. All Rights Reserved.
 #
 # By using this file, you agree to the terms and conditions set forth in
 # the LICENSE.html file which can be found at the top level of the ViewVC
@@ -19,8 +19,6 @@ import os
 import string
 import ConfigParser
 import fnmatch
-import re
-import vclib
 
 
 #########################################################################
@@ -40,28 +38,29 @@ import vclib
 #########################################################################
 
 class Config:
-  _sections = ('general', 'options', 'cvsdb', 'templates')
-  _force_multi_value = ('cvs_roots', 'forbidden', 'forbiddenre',
-                        'svn_roots', 'languages', 'kv_files',
-                        'root_parents')
+  _sections = ('general', 'utilities', 'options', 'cvsdb', 'templates')
+  _force_multi_value = ('cvs_roots', 'svn_roots', 'languages', 'kv_files',
+                        'root_parents', 'allowed_views', 'mime_types_files')
 
   def __init__(self):
     for section in self._sections:
       setattr(self, section, _sub_config())
 
-  def load_config(self, pathname, vhost=None):
+  def load_config(self, pathname, vhost=None, rootname=None):
     self.conf_path = os.path.isfile(pathname) and pathname or None
     self.base = os.path.dirname(pathname)
-
-    parser = ConfigParser.ConfigParser()
-    parser.read(pathname)
+    self.parser = ConfigParser.ConfigParser()
+    self.parser.read(self.conf_path or [])
 
     for section in self._sections:
-      if parser.has_section(section):
-        self._process_section(parser, section, section)
+      if self.parser.has_section(section):
+        self._process_section(self.parser, section, section)
 
-    if vhost and parser.has_section('vhosts'):
-      self._process_vhost(parser, vhost)
+    if vhost and self.parser.has_section('vhosts'):
+      self._process_vhost(self.parser, vhost)
+
+    if rootname:
+      self._process_root_options(self.parser, rootname)
 
   def load_kv_files(self, language):
     kv = _sub_config()
@@ -92,6 +91,10 @@ class Config:
 
     return kv
 
+  def path(self, path):
+    """Return path relative to the config file directory"""
+    return os.path.join(self.base, path)
+
   def _process_section(self, parser, section, subcfg_name):
     sc = getattr(self, subcfg_name)
 
@@ -111,22 +114,23 @@ class Config:
       setattr(sc, opt, value)
 
   def _process_vhost(self, parser, vhost):
+    # find a vhost name for this vhost, if any (if not, we've nothing to do)
     canon_vhost = self._find_canon_vhost(parser, vhost)
     if not canon_vhost:
-      # none of the vhost sections matched
       return
 
-    cv = canon_vhost + '-'
+    # overlay any option sections associated with this vhost name
+    cv = 'vhost-%s/' % (canon_vhost)
     lcv = len(cv)
     for section in parser.sections():
       if section[:lcv] == cv:
-        self._process_section(parser, section, section[lcv:])
+        base_section = section[lcv:]
+        if base_section not in self._sections:
+          raise IllegalOverrideSection('vhost', section)
+        self._process_section(parser, section, base_section)
 
   def _find_canon_vhost(self, parser, vhost):
-    vhost = string.lower(vhost)
-    # Strip (ignore) port number:
-    vhost = string.split(vhost, ':')[0]
-
+    vhost = string.split(string.lower(vhost), ':')[0]  # lower-case, no port
     for canon_vhost in parser.options('vhosts'):
       value = parser.get('vhosts', canon_vhost)
       patterns = map(string.lower, map(string.strip,
@@ -137,6 +141,55 @@ class Config:
 
     return None
 
+  def _process_root_options(self, parser, rootname):
+    rn = 'root-%s/' % (rootname)
+    lrn = len(rn)
+    for section in parser.sections():
+      if section[:lrn] == rn:
+        base_section = section[lrn:]
+        if base_section in self._sections:
+          if base_section == 'general':
+            raise IllegalOverrideSection('root', section)
+          self._process_section(parser, section, base_section)
+        elif _startswith(base_section, 'authz-'):
+          pass
+        else:
+          raise IllegalOverrideSection('root', section)
+          
+  def overlay_root_options(self, rootname):
+    "Overly per-root options atop the existing option set."
+    if not self.conf_path:
+      return
+    self._process_root_options(self.parser, rootname)
+
+  def _get_parser_items(self, parser, section):
+    """Basically implement ConfigParser.items() for pre-Python-2.3 versions."""
+    try:
+      return self.parser.items(section)
+    except AttributeError:
+      d = {}
+      for option in parser.options(section):
+        d[option] = parser.get(section, option)
+      return d.items()
+    
+  def get_authorizer_params(self, authorizer, rootname=None):
+    if not self.conf_path:
+      return {}
+
+    params = {}
+    authz_section = 'authz-%s' % (authorizer)
+    for section in self.parser.sections():
+      if section == authz_section:
+        for key, value in self._get_parser_items(self.parser, section):
+          params[key] = value
+    if rootname:
+      root_authz_section = 'root-%s/authz-%s' % (rootname, authorizer)
+      for section in self.parser.sections():
+        if section == root_authz_section:
+          for key, value in self._get_parser_items(self.parser, section):
+            params[key] = value
+    return params
+  
   def set_defaults(self):
     "Set some default values in the configuration."
 
@@ -144,28 +197,68 @@ class Config:
     self.general.svn_roots = { }
     self.general.root_parents = []
     self.general.default_root = ''
-    self.general.rcs_path = ''
-    if sys.platform == "win32":
-      self.general.cvsnt_exe_path = 'cvs'
-    else:
-      self.general.cvsnt_exe_path = None
-    self.general.use_rcsparse = 0
-    self.general.svn_path = ''
-    self.general.mime_types_file = ''
-    self.general.address = '<a href="mailto:user@insert.your.domain.here">No admin address has been configured</a>'
-    self.general.forbidden = []
-    self.general.forbiddenre = []
-    self.general.kv_files = []
+    self.general.mime_types_files = ["mimetypes.conf"]
+    self.general.address = ''
+    self.general.kv_files = [ ]
     self.general.languages = ['en-us']
 
+    self.utilities.rcs_dir = ''
+    if sys.platform == "win32":
+      self.utilities.cvsnt = 'cvs'
+    else:
+      self.utilities.cvsnt = None
+    self.utilities.svn = ''
+    self.utilities.diff = ''
+    self.utilities.cvsgraph = ''
+
+    self.options.root_as_url_component = 1
+    self.options.checkout_magic = 0
+    self.options.allowed_views = ['annotate', 'diff', 'markup', 'roots']
+    self.options.authorizer = None
+    self.options.mangle_email_addresses = 0
+    self.options.default_file_view = "log"
+    self.options.http_expiration_time = 600
+    self.options.generate_etags = 1
+    self.options.svn_ignore_mimetype = 0
+    self.options.svn_config_dir = None
+    self.options.use_rcsparse = 0
+    self.options.sort_by = 'file'
+    self.options.sort_group_dirs = 1
+    self.options.hide_attic = 1
+    self.options.hide_errorful_entries = 0
+    self.options.log_sort = 'date'
+    self.options.diff_format = 'h'
+    self.options.hide_cvsroot = 1
+    self.options.hr_breakable = 1
+    self.options.hr_funout = 1
+    self.options.hr_ignore_white = 0
+    self.options.hr_ignore_keyword_subst = 1
+    self.options.hr_intraline = 0
+    self.options.allow_compress = 0
+    self.options.template_dir = "templates"
+    self.options.docroot = None
+    self.options.show_subdir_lastmod = 0
+    self.options.show_logs = 1
+    self.options.show_log_in_markup = 1
+    self.options.cross_copies = 0
+    self.options.use_localtime = 0
+    self.options.short_log_len = 80
+    self.options.enable_syntax_coloration = 1
+    self.options.detect_encoding = 0
+    self.options.use_cvsgraph = 0
+    self.options.cvsgraph_conf = "cvsgraph.conf"
+    self.options.use_re_search = 0
+    self.options.dir_pagesize = 0
+    self.options.log_pagesize = 0
+    self.options.limit_changes = 100
+
+    self.templates.diff = None
     self.templates.directory = None
+    self.templates.error = None
+    self.templates.file = None
+    self.templates.graph = None
     self.templates.log = None
     self.templates.query = None
-    self.templates.diff = None
-    self.templates.graph = None
-    self.templates.annotate = None
-    self.templates.markup = None
-    self.templates.error = None
     self.templates.query_form = None
     self.templates.query_results = None
     self.templates.roots = None
@@ -180,110 +273,10 @@ class Config:
     self.cvsdb.readonly_passwd = '' 
     self.cvsdb.row_limit = 1000
     self.cvsdb.rss_row_limit = 100
+    self.cvsdb.check_database_for_root = 0
 
-    self.options.root_as_url_component = 0
-    self.options.default_file_view = "log"
-    self.options.checkout_magic = 0
-    self.options.sort_by = 'file'
-    self.options.sort_group_dirs = 1
-    self.options.hide_attic = 1
-    self.options.log_sort = 'date'
-    self.options.diff_format = 'h'
-    self.options.hide_cvsroot = 1
-    self.options.hr_breakable = 1
-    self.options.hr_funout = 1
-    self.options.hr_ignore_white = 1
-    self.options.hr_ignore_keyword_subst = 1
-    self.options.hr_intraline = 0
-    self.options.allow_annotate = 1
-    self.options.allow_markup = 1
-    self.options.allow_compress = 1
-    self.options.template_dir = "templates"
-    self.options.docroot = None
-    self.options.show_subdir_lastmod = 0
-    self.options.show_logs = 1
-    self.options.show_log_in_markup = 1
-    self.options.cross_copies = 0
-    self.options.py2html_path = '.'
-    self.options.short_log_len = 80
-    self.options.use_enscript = 0
-    self.options.enscript_path = ''
-    self.options.use_highlight = 0
-    self.options.highlight_path = ''
-    self.options.highlight_line_numbers = 1
-    self.options.highlight_convert_tabs = 2
-    self.options.use_php = 0
-    self.options.php_exe_path = 'php'
-    self.options.allow_tar = 0
-    self.options.use_cvsgraph = 0
-    self.options.cvsgraph_path = ''
-    self.options.cvsgraph_conf = "cvsgraph.conf"
-    self.options.use_re_search = 0
-    self.options.use_pagesize = 0
-    self.options.limit_changes = 100
-    self.options.use_localtime = 0
-    self.options.http_expiration_time = 600
-    self.options.generate_etags = 1
-
-  def is_forbidden(self, root, path_parts, pathtype):
-    # If we don't have a root and path to check, get outta here.
-    if not (root and path_parts):
-      return 0
-
-    # Give precedence to the new 'forbiddenre' stuff first.
-    if self.general.forbiddenre:
-
-      # Join the root and path-parts together into one path-like thing.
-      root_and_path = string.join([root] + path_parts, "/")
-      if pathtype == vclib.DIR:
-        root_and_path = root_and_path + '/'
-      
-      # If we still have a list of strings, replace those suckers with
-      # lists of (compiled_regex, negation_flag)
-      if type(self.general.forbiddenre[0]) == type(""):
-        for i in range(len(self.general.forbiddenre)):
-          pat = self.general.forbiddenre[i]
-          if pat[0] == '!':
-            self.general.forbiddenre[i] = (re.compile(pat[1:]), 1)
-          else:
-            self.general.forbiddenre[i] = (re.compile(pat), 0)
-
-      # Do the forbiddenness test.
-      default = 0
-      for (pat, negated) in self.general.forbiddenre:
-        match = pat.search(root_and_path)
-        if negated:
-          default = 1
-          if match:
-            return 0
-        elif match:
-          return 1
-      return default
-
-    # If no 'forbiddenre' is in use, we check 'forbidden', which only
-    # looks at the top-most directory.
-    elif self.general.forbidden:
-
-      # A root and a single non-directory path component?  That's not
-      # a module.
-      if len(path_parts) == 1 and pathtype != vclib.DIR:
-        return 0
-      
-      # Do the forbiddenness test.
-      module = path_parts[0]
-      default = 0
-      for pat in self.general.forbidden:
-        if pat[0] == '!':
-          default = 1
-          if fnmatch.fnmatchcase(module, pat[1:]):
-            return 0
-        elif fnmatch.fnmatchcase(module, pat):
-          return 1
-      return default
-
-    # No forbiddenness configuration?  Just allow it.
-    else:
-      return 0
+def _startswith(somestr, substr):
+  return somestr[:len(substr)] == substr
 
 def _parse_roots(config_name, config_value):
   roots = { }
@@ -295,8 +288,18 @@ def _parse_roots(config_name, config_value):
     roots[name] = path
   return roots
 
+class ViewVCConfigurationError(Exception):
+  pass
 
-class MalformedRoot(Exception):
+class IllegalOverrideSection(ViewVCConfigurationError):
+  def __init__(self, override_type, section_name):
+    self.section_name = section_name
+    self.override_type = override_type
+  def __str__(self):
+    return "malformed configuration: illegal %s override section: %s" \
+           % (self.override_type, self.section_name)
+  
+class MalformedRoot(ViewVCConfigurationError):
   def __init__(self, config_name, value_given):
     Exception.__init__(self, config_name, value_given)
     self.config_name = config_name
