@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*-python-*-
 #
-# Copyright (C) 1999-2008 The ViewCVS Group. All Rights Reserved.
+# Copyright (C) 1999-2009 The ViewCVS Group. All Rights Reserved.
 #
 # By using this file, you agree to the terms and conditions set forth in
 # the LICENSE.html file which can be found at the top level of the ViewVC
@@ -25,10 +25,10 @@ import time
 
 import cvsdb
 import viewvc
-import vclib
 import ezt
 import debug
 import urllib
+import fnmatch
 
 class FormData:
     def __init__(self, form):
@@ -274,6 +274,20 @@ def prev_rev(rev):
         r = r[:-2]
     return string.join(r, '.')
 
+def is_forbidden(cfg, cvsroot_name, module):
+    auth_params = cfg.get_authorizer_params('forbidden', cvsroot_name)
+    forbidden = auth_params.get('forbidden', '')
+    forbidden = map(string.strip, filter(None, string.split(forbidden, ',')))
+    default = 0
+    for pat in forbidden:
+        if pat[0] == '!':
+            default = 1
+            if fnmatch.fnmatchcase(module, pat[1:]):
+                return 0
+        elif fnmatch.fnmatchcase(module, pat):
+            return 1
+    return default
+    
 def build_commit(server, cfg, desc, files, cvsroots, viewvc_link):
     ob = _item(num_files=len(files), files=[])
     
@@ -283,8 +297,19 @@ def build_commit(server, cfg, desc, files, cvsroots, viewvc_link):
         ob.log = '&nbsp;'
 
     for commit in files:
-        parts = filter(None, string.split(commit.GetDirectory(), '/'))
-        if parts and cfg.options.hide_cvsroot and parts[0] == 'CVSROOT':
+        repository = commit.GetRepository()
+        directory = commit.GetDirectory()
+        cvsroot_name = cvsroots.get(repository)
+
+        ## find the module name (if any)
+        try:
+            module = filter(None, string.split(directory, '/'))[0]
+        except IndexError:
+            module = None
+
+        ## skip commits we aren't supposed to show
+        if module and ((module == 'CVSROOT' and cfg.options.hide_cvsroot) \
+                       or is_forbidden(cfg, cvsroot_name, module)):
             continue
 
         ctime = commit.GetTime()
@@ -295,18 +320,13 @@ def build_commit(server, cfg, desc, files, cvsroots, viewvc_link):
             ctime = time.strftime("%y/%m/%d %H:%M %Z", time.localtime(ctime))
           else:
             ctime = time.strftime("%y/%m/%d %H:%M", time.gmtime(ctime)) \
-                  + ' UTC'
+                    + ' UTC'
         
         ## make the file link
-        repository = commit.GetRepository()
-        directory = commit.GetDirectory()
-        file = (directory and directory + "/") + commit.GetFile()
-        cvsroot_name = cvsroots.get(repository)
-
-        ## skip forbidden files
-        if cfg.is_forbidden(cvsroot_name,
-                            filter(None, string.split(file, "/")), vclib.FILE):
-            continue
+        try:
+            file = (directory and directory + "/") + commit.GetFile()
+        except:
+            raise Exception, str([directory, commit.GetFile()])
 
         ## if we couldn't find the cvsroot path configured in the 
         ## viewvc.conf file, then don't make the link
@@ -392,13 +412,10 @@ def main(server, cfg, viewvc_link):
         commits = [ ]
         query = 'skipped'
 
-    script_name = server.getenv('SCRIPT_NAME', '')
-
-    data = {
+    data = ezt.TemplateData({
       'cfg' : cfg,
       'address' : cfg.general.address,
       'vsn' : viewvc.__version__,
-
       'repository' : server.escape(form_data.repository, 1),
       'branch' : server.escape(form_data.branch, 1),
       'directory' : server.escape(form_data.directory, 1),
@@ -415,18 +432,13 @@ def main(server, cfg, viewvc_link):
       'commits' : commits,
       'num_commits' : len(commits),
       'rss_href' : None,
-      }
-
-    if form_data.hours:
-      data['hours'] = form_data.hours
-    else:
-      data['hours'] = 2
-
-    server.header()
+      'hours' : form_data.hours and form_data.hours or 2,
+      })
 
     # generate the page
+    server.header()
     template = viewvc.get_view_template(cfg, "query")
-    template.generate(sys.stdout, data)
+    template.generate(server.file(), data)
 
   except SystemExit, e:
     pass
